@@ -1,9 +1,9 @@
 /*
- * "$Id: sysman.c 9824 2011-06-10 23:41:28Z mike $"
+ * "$Id: sysman.c 11106 2013-07-08 12:29:10Z msweet $"
  *
  *   System management functions for the CUPS scheduler.
  *
- *   Copyright 2007-2011 by Apple Inc.
+ *   Copyright 2007-2013 by Apple Inc.
  *   Copyright 2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -44,6 +44,8 @@
 #  include <IOKit/pwr_mgt/IOPMLib.h>
 #  ifdef HAVE_IOKIT_PWR_MGT_IOPMLIBPRIVATE_H
 #    include <IOKit/pwr_mgt/IOPMLibPrivate.h>
+#  else
+#    define kIOPMAssertionTypeDenySystemSleep CFSTR("DenySystemSleep")
 #  endif /* HAVE_IOKIT_PWR_MGT_IOPMLIBPRIVATE_H */
 #endif /* __APPLE__ */
 
@@ -69,9 +71,9 @@
  * Local globals...
  */
 
-#ifdef kIOPMAssertionTypeDenySystemSleep
-static IOPMAssertionID	dark_wake = 0;	/* "Dark wake" assertion for sharing */
-#endif /* kIOPMAssertionTypeDenySystemSleep */
+#if defined(kIOPMAssertionTypeDenySystemSleep) || defined(kIOPMAssertNetworkClientActive)
+static IOPMAssertionID	keep_awake = 0;	/* Keep the system awake while printing */
+#endif /* kIOPMAssertionTypeDenySystemSleep || kIOPMAssertNetworkClientActive */
 
 
 /*
@@ -87,9 +89,6 @@ cupsdCleanDirty(void)
   if (DirtyFiles & CUPSD_DIRTY_CLASSES)
     cupsdSaveAllClasses();
 
-  if (DirtyFiles & CUPSD_DIRTY_REMOTE)
-    cupsdSaveRemoteCache();
-
   if (DirtyFiles & CUPSD_DIRTY_PRINTCAP)
     cupsdWritePrintcap();
 
@@ -123,10 +122,9 @@ cupsdCleanDirty(void)
 void
 cupsdMarkDirty(int what)		/* I - What file(s) are dirty? */
 {
-  cupsdLogMessage(CUPSD_LOG_DEBUG, "cupsdMarkDirty(%c%c%c%c%c%c)",
+  cupsdLogMessage(CUPSD_LOG_DEBUG, "cupsdMarkDirty(%c%c%c%c%c)",
 		  (what & CUPSD_DIRTY_PRINTERS) ? 'P' : '-',
 		  (what & CUPSD_DIRTY_CLASSES) ? 'C' : '-',
-		  (what & CUPSD_DIRTY_REMOTE) ? 'R' : '-',
 		  (what & CUPSD_DIRTY_PRINTCAP) ? 'p' : '-',
 		  (what & CUPSD_DIRTY_JOBS) ? 'J' : '-',
 		  (what & CUPSD_DIRTY_SUBSCRIPTIONS) ? 'S' : '-');
@@ -219,21 +217,32 @@ cupsdSetBusyState(void)
 #endif /* HAVE_VPROC_TRANSACTION_BEGIN */
   }
 
-#ifdef kIOPMAssertionTypeDenySystemSleep
-  if (cupsArrayCount(PrintingJobs) > 0 && !dark_wake)
+#if defined(kIOPMAssertionTypeDenySystemSleep) || defined(kIOPMAssertNetworkClientActive)
+  if (cupsArrayCount(PrintingJobs) > 0 && !keep_awake)
   {
-    cupsdLogMessage(CUPSD_LOG_DEBUG, "Asserting dark wake.");
+#  ifdef kIOPMAssertNetworkClientActive
+    cupsdLogMessage(CUPSD_LOG_DEBUG, "Asserting NetworkClientActive.");
+
+    IOPMAssertionCreateWithName(kIOPMAssertNetworkClientActive,
+				kIOPMAssertionLevelOn,
+				CFSTR("org.cups.cupsd"), &keep_awake);
+
+#  else
+    cupsdLogMessage(CUPSD_LOG_DEBUG, "Asserting DenySystemSleep.");
+
     IOPMAssertionCreateWithName(kIOPMAssertionTypeDenySystemSleep,
 				kIOPMAssertionLevelOn,
-				CFSTR("org.cups.cupsd"), &dark_wake);
+				CFSTR("org.cups.cupsd"), &keep_awake);
+
+#  endif /* kIOPMAssertNetworkClientActive */
   }
-  else if (cupsArrayCount(PrintingJobs) == 0 && dark_wake)
+  else if (cupsArrayCount(PrintingJobs) == 0 && keep_awake)
   {
-    cupsdLogMessage(CUPSD_LOG_DEBUG, "Releasing dark wake assertion.");
-    IOPMAssertionRelease(dark_wake);
-    dark_wake = 0;
+    cupsdLogMessage(CUPSD_LOG_DEBUG, "Releasing power assertion.");
+    IOPMAssertionRelease(keep_awake);
+    keep_awake = 0;
   }
-#endif /* kIOPMAssertionTypeDenySystemSleep */
+#endif /* kIOPMAssertionTypeDenySystemSleep || kIOPMAssertNetworkClientActive */
 }
 
 
@@ -266,8 +275,8 @@ cupsdSetBusyState(void)
 #  define SYSEVENT_NAMECHANGED	0x10	/* Computer name changed */
 
 
-/* 
- * Structures... 
+/*
+ * Structures...
  */
 
 typedef struct cupsd_sysevent_s		/*** System event data ****/
@@ -286,14 +295,14 @@ typedef struct cupsd_thread_data_s	/*** Thread context data  ****/
 } cupsd_thread_data_t;
 
 
-/* 
- * Local globals... 
+/*
+ * Local globals...
  */
 
 static pthread_t	SysEventThread = NULL;
 					/* Thread to host a runloop */
 static pthread_mutex_t	SysEventThreadMutex = { 0 };
-					/* Coordinates access to shared gloabals */ 
+					/* Coordinates access to shared gloabals */
 static pthread_cond_t	SysEventThreadCond = { 0 };
 					/* Thread initialization complete condition */
 static CFRunLoopRef	SysEventRunloop = NULL;
@@ -316,8 +325,8 @@ static CFStringRef	ComputerNameKey = NULL,
 static cupsd_sysevent_t	LastSysEvent;	/* Last system event (for delayed sleep) */
 
 
-/* 
- * Local functions... 
+/*
+ * Local functions...
  */
 
 static void	*sysEventThreadEntry(void);
@@ -499,8 +508,8 @@ sysEventThreadEntry(void)
 						   kSCEntNetIPv6);
 
   if (!NetworkGlobalKeyDNS)
-    NetworkGlobalKeyDNS = 
-	SCDynamicStoreKeyCreateNetworkGlobalEntity(kCFAllocatorDefault, 
+    NetworkGlobalKeyDNS =
+	SCDynamicStoreKeyCreateNetworkGlobalEntity(kCFAllocatorDefault,
 						   kSCDynamicStoreDomainState,
 						   kSCEntNetDNS);
 
@@ -582,7 +591,7 @@ sysEventThreadEntry(void)
 
   threadData.timerRef =
       CFRunLoopTimerCreate(kCFAllocatorDefault,
-                           CFAbsoluteTimeGetCurrent() + (86400L * 365L * 10L), 
+                           CFAbsoluteTimeGetCurrent() + (86400L * 365L * 10L),
 			   86400L * 365L * 10L, 0, 0, sysEventTimerNotifier,
 			   &timerContext);
   CFRunLoopAddTimer(CFRunLoopGetCurrent(), threadData.timerRef,
@@ -674,7 +683,7 @@ sysEventPowerNotifier(
 	break;
 
     case kIOMessageSystemHasPoweredOn:
-       /* 
+       /*
 	* Because powered on is followed by a net-changed event, delay
 	* before sending it.
 	*/
@@ -702,7 +711,7 @@ sysEventPowerNotifier(
 
     if (sendit == 1)
     {
-     /* 
+     /*
       * Send the event to the main thread now.
       */
 
@@ -712,7 +721,7 @@ sysEventPowerNotifier(
     }
     else
     {
-     /* 
+     /*
       * Send the event to the main thread after 1 to 2 seconds.
       */
 
@@ -738,7 +747,7 @@ sysEventConfigurationNotifier(
 
 
   threadData = (cupsd_thread_data_t *)context;
-  
+
   (void)store;				/* anti-compiler-warning-code */
 
   CFRange range = CFRangeMake(0, CFArrayGetCount(changedKeys));
@@ -758,12 +767,12 @@ sysEventConfigurationNotifier(
   }
 
  /*
-  * Because we registered for several different kinds of change notifications 
-  * this callback usually gets called several times in a row. We use a timer to 
+  * Because we registered for several different kinds of change notifications
+  * this callback usually gets called several times in a row. We use a timer to
   * de-bounce these so we only end up generating one event for the main thread.
   */
 
-  CFRunLoopTimerSetNextFireDate(threadData->timerRef, 
+  CFRunLoopTimerSetNextFireDate(threadData->timerRef,
   				CFAbsoluteTimeGetCurrent() + 5);
 }
 
@@ -865,33 +874,27 @@ sysUpdate(void)
            p;
 	   p = (cupsd_printer_t *)cupsArrayNext(Printers))
       {
-	if (p->type & CUPS_PRINTER_DISCOVERED)
-	{
-	  cupsdLogMessage(CUPSD_LOG_DEBUG,
-	                  "Deleting remote destination \"%s\"", p->name);
-	  cupsArraySave(Printers);
-	  cupsdDeletePrinter(p, 0);
-	  cupsArrayRestore(Printers);
-	}
-	else
-	{
-	  cupsdLogMessage(CUPSD_LOG_DEBUG,
-	                  "Deregistering local printer \"%s\"", p->name);
-	  cupsdDeregisterPrinter(p, 0);
-	}
+	cupsdLogMessage(CUPSD_LOG_DEBUG,
+			"Deregistering local printer \"%s\"", p->name);
+	cupsdDeregisterPrinter(p, 0);
       }
 
       cupsdCleanDirty();
 
 #ifdef kIOPMAssertionTypeDenySystemSleep
      /*
-      * Tell the OS it is OK to sleep when we remove our assertion...
+      * Remove our assertion as needed since the user wants the system to
+      * sleep (different than idle sleep)...
       */
 
-      IOAllowPowerChange(sysevent.powerKernelPort,
-                         sysevent.powerNotificationID);
+      if (keep_awake)
+      {
+	cupsdLogMessage(CUPSD_LOG_DEBUG, "Releasing dark wake assertion.");
+	IOPMAssertionRelease(keep_awake);
+	keep_awake = 0;
+      }
+#endif /* kIOPMAssertionTypeDenySystemSleep */
 
-#else
      /*
       * If we have no printing jobs, allow the power change immediately.
       * Otherwise set the SleepJobs time to 15 seconds in the future when
@@ -935,7 +938,6 @@ sysUpdate(void)
 			     sysevent.powerNotificationID);
 	}
       }
-#endif /* kIOPMAssertionTypeDenySystemSleep */
     }
 
     if (sysevent.event & SYSEVENT_WOKE)
@@ -944,29 +946,25 @@ sysUpdate(void)
       IOAllowPowerChange(sysevent.powerKernelPort,
                          sysevent.powerNotificationID);
       Sleeping = 0;
+
+#ifdef kIOPMAssertionTypeDenySystemSleep
+      if (cupsArrayCount(PrintingJobs) > 0 && !keep_awake)
+      {
+	cupsdLogMessage(CUPSD_LOG_DEBUG, "Asserting dark wake.");
+	IOPMAssertionCreateWithName(kIOPMAssertionTypeDenySystemSleep,
+				    kIOPMAssertionLevelOn,
+				    CFSTR("org.cups.cupsd"), &keep_awake);
+      }
+#endif /* kIOPMAssertionTypeDenySystemSleep */
+
       cupsdCheckJobs();
     }
 
     if (sysevent.event & SYSEVENT_NETCHANGED)
     {
       if (!Sleeping)
-      {
         cupsdLogMessage(CUPSD_LOG_DEBUG,
 	                "System network configuration changed");
-
-       /*
-        * Resetting browse_time before calling cupsdSendBrowseList causes
-	* browse packets to be sent for local shared printers.
-        */
-
-	for (p = (cupsd_printer_t *)cupsArrayFirst(Printers);
-	     p;
-	     p = (cupsd_printer_t *)cupsArrayNext(Printers))
-	  p->browse_time = 0;
-
-        cupsdSendBrowseList();
-	cupsdRestartPolling();
-      }
       else
         cupsdLogMessage(CUPSD_LOG_DEBUG,
 	                "System network configuration changed; "
@@ -989,11 +987,13 @@ sysUpdate(void)
 	     p = (cupsd_printer_t *)cupsArrayNext(Printers))
 	  cupsdDeregisterPrinter(p, 1);
 
+#  if defined(HAVE_DNSSD) || defined(HAVE_AVAHI)
        /*
         * Update the computer name and BTMM domain list...
 	*/
 
 	cupsdUpdateDNSSDName();
+#  endif /* HAVE_DNSSD || HAVE_AVAHI */
 
        /*
 	* Now re-register them...
@@ -1002,10 +1002,7 @@ sysUpdate(void)
 	for (p = (cupsd_printer_t *)cupsArrayFirst(Printers);
 	     p;
 	     p = (cupsd_printer_t *)cupsArrayNext(Printers))
-	{
-	  p->browse_time = 0;
 	  cupsdRegisterPrinter(p);
-	}
       }
       else
         cupsdLogMessage(CUPSD_LOG_DEBUG,
@@ -1018,5 +1015,5 @@ sysUpdate(void)
 
 
 /*
- * End of "$Id: sysman.c 9824 2011-06-10 23:41:28Z mike $".
+ * End of "$Id: sysman.c 11106 2013-07-08 12:29:10Z msweet $".
  */
